package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.dto.QcCasesAllDTO;
import com.diagbot.dto.QcCasesEntryFindDTO;
import com.diagbot.dto.QcCasesEntryHospitalDTO;
import com.diagbot.entity.QcCasesEntryHospital;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.QcCasesEntryHospitalService;
import com.diagbot.service.impl.QcCasesEntryHospitalServiceImpl;
import com.diagbot.util.DateUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.util.SysUserUtils;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.QcCasesEntryFindVO;
import com.diagbot.vo.QcCasesEntryHospitalVO;
import com.diagbot.vo.QcCasesEntryUpdataVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-04-13 16:23
 */
@Component
public class QcCasesEntryHospitalFacade extends QcCasesEntryHospitalServiceImpl {
    @Autowired
    QcCasesEntryHospitalService qcCasesEntryHospitalService;
    @Autowired
    private QcCasesFacade qcCasesFacade;

    /**
     *
     * @param qcCasesEntryHospitalVO
     * @return
     */
    public List<QcCasesEntryHospitalDTO> QcCasesEntryHospitals(QcCasesEntryHospitalVO qcCasesEntryHospitalVO) {
        List<QcCasesEntryHospitalDTO> data = qcCasesEntryHospitalService.getQcCasesEntryAlls(qcCasesEntryHospitalVO);
        return data;
    }

    /**
     *
     * @param qcCasesEntryUpdataVO
     * @return
     */
    public boolean updataQcCasesEntrys(QcCasesEntryUpdataVO qcCasesEntryUpdataVO) {
        //1.先判断数据是否存在有效
        QueryWrapper<QcCasesEntryHospital> qc = new QueryWrapper<>();
        qc.eq("id", qcCasesEntryUpdataVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(SysUserUtils.getCurrentHospitalID()));
        int sum = count(qc);
        if (sum == 0) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "该条目不存在");
        }
        UpdateWrapper<QcCasesEntryHospital> qcCasesEntry = new UpdateWrapper<>();
        qcCasesEntry.eq("id", qcCasesEntryUpdataVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("hospital_id", Long.valueOf(SysUserUtils.getCurrentHospitalID()))
                .set("modifier", SysUserUtils.getCurrentPrincipleID())
                .set("gmt_modified", DateUtil.now());
        qcCasesEntry.set(qcCasesEntryUpdataVO.getScore() != null,
                "score", qcCasesEntryUpdataVO.getScore());
        qcCasesEntry.set(qcCasesEntryUpdataVO.getMsg() != null,
                "msg", qcCasesEntryUpdataVO.getMsg());
        qcCasesEntry.set(qcCasesEntryUpdataVO.getIsUsed() != null,
                "is_used", qcCasesEntryUpdataVO.getIsUsed());
        qcCasesEntry.set(qcCasesEntryUpdataVO.getIsReject() != null,
                "is_reject", qcCasesEntryUpdataVO.getIsReject());
        return update(new QcCasesEntryHospital(), qcCasesEntry);
    }

    /**
     *
     * @return
     */
    public List<QcCasesAllDTO> getQcCasesAlls() {
        return  qcCasesFacade.getQcCasesAlls();
    }

    /**
     *
     * @param qcCasesEntryFindVO
     * @return
     */
    public List<QcCasesEntryFindDTO> findQcCasesEntryAlls(QcCasesEntryFindVO qcCasesEntryFindVO) {
        List<QcCasesEntryFindDTO> data = qcCasesEntryHospitalService.findQcCasesEntryAll(qcCasesEntryFindVO);
        return data;
    }
}
